package de.w665.sharepulse.db.repo;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.gson.Gson;
import com.rethinkdb.RethinkDB;
import com.rethinkdb.net.Connection;
import com.rethinkdb.net.Result;
import de.w665.sharepulse.db.RethinkDBConfig;
import de.w665.sharepulse.db.RethinkDBConnector;
import de.w665.sharepulse.model.UserLogin;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

@Slf4j
@Repository
public class UserLoginRepository {
    private final RethinkDB r;
    private final Connection connection;
    private final RethinkDBConfig config;
    private final String TABLE_NAME = "user_logins";

    private final Gson gson = new Gson();
    private final ObjectMapper mapper = new ObjectMapper();


    @Autowired
    public UserLoginRepository(RethinkDBConnector connector, RethinkDBConfig config) {
        this.r = connector.getR();
        this.connection = connector.getConnection();
        this.config = config;
    }

    public void insertUserLogin(UserLogin userLogin) {
        String uuid = r.uuid().run(connection, String.class).first();
        userLogin.setId(uuid);
        r.db(config.getDatabase()).table(TABLE_NAME).insert(userLogin).run(connection);
    }

    public UserLogin getLastLogin(String userId) {
        // Get the second most recent login (the most recent is the current one)
        Result<UserLogin> result = r.db(config.getDatabase()).table(TABLE_NAME)
                .orderBy().optArg("index", r.desc("loginTime"))
                .filter(r.hashMap("userId", userId))
                .skip(1).limit(1)
                .run(connection, UserLogin.class);
        // Return the second most recent login if exists
        return result.hasNext() ? result.next() : null;
    }
}
